<?php
/****************************************************
*      e-Cart class for e-commerce web site         *
*  *********************************************    *
*  Author:   Riccardo Sadocchi                      *
*  License:  OpenSource - free                      *
****************************************************/


// definisce la path dove vengono salvati i log del carrello
define('CART_LOG_PTH','cart-log/');
if(!file_exists(CART_LOG_PTH)){ mkdir(CART_LOG_PTH); }

/*
    Si può cambiare la valuta utlizzando il formato a 3 lettere maiuscole
    es. EUR, USD, GBP, CHF, RUB, AUD ...

    il 'fileName' serve ad esempio a recuperare un carrello lascito in sospeso,
    o magari passando tra le diverse pagine del sito
*/


/***************************************************
*  **********************************************  *
*                   START CLASS                    *
*  **********************************************  *
***************************************************/

class Cart
{
    public $tax;              // % tasse da applicare
    public $pay_method;       // metodo di pagamento
    public $fileName;         // qui viene memorizzato il nome del file temporaneo
    
    protected $code;          // codice univoco del carrello
    protected $cv;            // valuta
    protected $change;        // cambio
    protected $start;         // data di creazione carrello, formato aaaa-mm-dd
    protected $userID;        // user id
    protected $tot_amount;    // totale carrello senza tasse
    protected $tot_tax;       // totale tasse carrello
    protected $items;         // array di tutti gli articoli (a loro volta array)

    /***************************************************
    *                 PUBLIC METHOD                    *
    ***************************************************/
    public function Cart($userID)
    {
        $this->userID = $userID;
        $this->start = date('Y-m-d');
        $this->cv = 'EUR';
        $this->tax = 0;
        $this->change = 1;
        $this->pay_method = 'undefined';
        $this->items = array();
        $this->code = 'temporary';
    }

    // aggiunge un articolo
    public function add($art,$desc,$price,$qty,$var=null)
    {
        $pos = count($this->items);
        // cosctruisco l'articolo
        $item['pos'] = $pos;
        $item['art'] = $art;
        $item['desc'] = $desc;
        $item['price'] = $price;
        $item['qty'] = $qty;
        foreach($var as $k=>$v){$item[$k] = $v;}
        // lo aggiungo al carrello se non ce n'è uno esattamente uguale
        if(!in_array($item, $this->items)){$this->items[] = $item;}
        else{die("Attenzione! Nel carrello &grave; gi&agrave; presente un articolo identico!!!");}
        // aggiorno i totali
        $this->totals();
        // salvo il temporaneo su xml
        $this->temp_cart2xml();
    }

    // aggiorna un articolo
    public function update($item)
    {
        $pos = $item['pos'];
        $this->items[$pos] = $item;
        // aggiorno i totali
        $this->totals();
        // salvo il temporaneo su xml
        $this->temp_cart2xml();
    }

    // rimuove un articolo
    public function remove($pos)
    {
        // rimuovo l'articolo
        $this->items[$pos] = null;
        array_splice($this->items, $pos, 1);
        array_values($this->items);
        $this->reIndexPos();
        // aggiorno i totali
        $this->totals();
        // salvo il temporaneo su xml
        $this->temp_cart2xml();
    }

    // cancella tutti gli articoli ma mantiene l'oggetto carrello
    public function clear()
    {
        $this->items = null;
        unset($this->items);
        // azzero i totali
        $this->tot_amount = 0;
        $this->tot_tax = 0;
        // salvo il temporaneo su xml
        $this->temp_cart2xml();
    }

    // elimina tutto il carrello
    public function delete()
    {
        $this->clear();
        unset($this->cv);
        unset($this->start);
        unset($this->userID);
        unset($this->tax);
        unset($this->change);
        // cancello il file xml temporaneo e poi azzero la variabile
        unlink($this->fileName);
        unset($this->fileName);
    }

    // cambia la valuta del carrello
    public function change_currency($currency)
    {
        strtoupper($currency);
        if($currency == $this->cv){return;}
        elseif($currency == "EUR" && $currency != $this->cv){$latest = 1;}
        else{$latest = (double)$this->latest_rate($currency);}
        // cambio tutti i prezzi e totali
        $tot = count($this->items);
        for($i=0; $i<$tot; $i++){$this->items[$i]['price'] = (double)($this->items[$i]['price'] * $latest);}
        //
        if($latest != 1){$this->cv = $currency;}
        $this->change = $latest;
        $this->totals();
        // salvo il temporaneo su xml
        $this->temp_cart2xml();
    }

    // ricreo il carrello leggendo il file xml temporaneo
    public function recoveryCart($which)
    {
        $XML = simplexml_load_file($which);
        // dati generici carrello
        foreach($XML->xpath('/root/cart') as $head)
        {
            $this->code = (string)$head['code'];
            $this->userID = (string)$head->userID;
            $this->start = (string)$head->date;
            $this->cv = (string)$head->currency;
            $this->change = (double)$head->change;
            $this->tax = (double)$head->tax;
            $this->tot_amount = (double)$head->tot_amount;
            $this->tot_tax = (double)$head->tot_tax;
            $this->fileName = (string)$head->filename;
        }
        // dati degli articoli
        foreach($XML->xpath('/root/items/item') as $body)
        {
            unset($item);
            $item['pos'] = (int)$body->pos;
            $item['art'] = (string)$body->art;
            $item['desc'] = (string)$body->desc;
            $item['price'] = (double)$body->price;
            $item['qty'] = (int)$body->qty;
            $item['row_amount'] = (double)$body->row_amount;
            $item['row_tax'] = (double)$body->row_tax;
            $know = array('pos','art','desc','price','qty','row_amount','row_tax');
            foreach($body as $k=>$v){if(!in_array($k,$know)){$item[$k] = (string)$body->$k;}}
            $this->items[] = $item;
        }
    }

    // mostra il carrello
    public function showcart()
    { /* da implementare esternamente in base alle proprie esigenze */ }

    // salva il carrello acquistato in un file
    public function cart2file()
    { /* da implementare esternamente in base alle proprie esigenze */ }

    // salva il carrello su mysql
    public function cart2db()
    { /* da implementare esternamente in base alle proprie esigenze */ }


    /***************************************************
    *                 PROTECTED METHOD                 *
    ***************************************************/
    // recupera il cambio aggiornato della valuta scelta da ecb.europa (sempre base euro)
    protected function latest_rate($cur)
    { /* http://www.ecb.europa.eu/stats/exchange/eurofxref/html/index.en.html#latest */
        $latestRate = array();
        $latestCur = array();
        $XML = simplexml_load_file("http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml");
        foreach($XML->Cube->Cube->Cube as $rate)
        {
            $latestCur[] = $rate['currency'];
            $latestRate[] = $rate['rate'];
        }
        if(in_array($cur, $latestCur))
        {
            $i = array_search($cur,$latestCur);
            $latest = $latestRate[$i];
        }
        else{$latest = 1;}
        return $latest;
    }

    // calcola il totale del carrello tenedo separate le tasse
    protected function totals()
    {
        $this->tot_amount = 0.00;
        $this->tot_tax = 0.00;
        $t = count($this->items);
        for($i=0; $i<$t; $i++)
        {
            $this->items[$i]['row_amount'] = ($this->items[$i]['price'] * $this->items[$i]['qty']);
            $this->tot_amount += $this->items[$i]['row_amount'];
            if($this->tax != 0)
            {
                $this->items[$i]['row_tax'] = ($this->items[$i]['price'] * $this->items[$i]['qty']) * ($this->tax / 100);
                $this->tot_tax += $this->items[$i]['row_tax'];
            }
            else
            {
                unset($this->items[$i]['row_tax']);
            }
        }
    }
    
    // riallinea posizione in items con 'pos' in item dopo l'eliminazione di un articolo
    protected function reIndexPos()
    {
        $tot = count($this->items);
        for($i=0; $i<$tot; $i++){$this->items[$i]['pos'] = $i;}
    }
    
    // crea un codice alfanumerico casuale di 10 cifre
    protected function unicod($n=10)
    {
        $unicod = "";
        for($i=0; $i<$n; $i++)
        {
            do{$c = ceil(rand(48,122));}while(!((($c >= 48)&&($c <= 57))||(($c >= 65)&&($c <= 90))||(($c >= 97)&&($c <= 122))));
            $unicod = $unicod.chr($c);
        }
        return $unicod;
    }

    // salva il carrello in un file xml temporaneo per il passaggio tra le pagine
    protected function temp_cart2xml()
    {
        $path = CART_LOG_PTH."temp/";
        if(!file_exists($path)){mkdir($path);}
        if($this->code == 'temporary'){$this->code = $this->unicod();}
        $name = $this->start."_".$this->userID."_in-progress.xml";
        $this->fileName = $path.$name;
        // parti fisse del file
        $head = "<?xml version='1.0' encoding='UTF-8'?>\n\n<root>\n\n";
        $head .= "<cart code='$this->code'>\n";
        $head .= " <date>$this->start</date>\n";
        $head .= " <userID>$this->userID</userID>\n";
        $head .= " <currency>$this->cv</currency>\n";
        $head .= " <change>$this->change</change>\n";
        $head .= " <tax>$this->tax</tax>\n";
        $head .= " <tot_amount>$this->tot_amount</tot_amount>\n";
        $head .= " <tot_tax>$this->tot_tax</tot_tax>\n";
        $head .= " <filename>$this->fileName</filename>\n";
        $head .= "</cart>\n\n<items>\n";
        $foot = "</items>\n\n</root>\n";
        // inizio procedure scrittura del file (tutte le volte lo apro, pulisco e riscrivo)
        $n = count($this->items);
        // array degli attributi base di ogni articolo
        $know = array('pos','art','desc','price','qty','row_amount','row_tax');
        try
        {
            // creo il file, se esiste lo apro cancellando il contenuto
            $file = fopen($this->fileName,'w');
            // inizio la scrittura
            fwrite($file,$head);
            for($i=0; $i<$n; $i++)
            {
                // costruisco il blocco dell'articolo
                $item = " <item>\n";
                $item .= "  <pos>$i</pos>\n";
                $item .= "  <art>".$this->items[$i]['art']."</art>\n";
                $item .= "  <desc>".$this->items[$i]['desc']."</desc>\n";
                $item .= "  <price>".$this->items[$i]['price']."</price>\n";
                $item .= "  <qty>".$this->items[$i]['qty']."</qty>\n";
                $item .= "  <row_amount>".$this->items[$i]['row_amount']."</row_amount>\n";
                $item .= "  <row_tax>".$this->items[$i]['row_tax']."</row_tax>\n";
                foreach($this->items[$i] as $k=>$v){if(!in_array($k,$know)){$item .= "  <$k>$v</$k>\n";}}
                $item .= " </item>\n";
                // scrivo il blocco dell'articolo
                fwrite($file,$item);
            }
            // scrivo il footer del file
            fwrite($file,$foot);
            // chiudo il file
            fclose($file);
        }
        catch(Exception $e)
        {
            echo "ERRORE! Impossibile scrivere il file temporaneo del carrello!!!<br>".$e->getMessage();
            exit();
        }
    }
}
/***************************************************
*  **********************************************  *
*                     END CLASS                    *
*  **********************************************  *
***************************************************/





// esempio di estensione classe Cart per mostrare a video il carrello
class MyCart extends Cart
{
    // tabella carrello con bootstrap
    public function showcart()
    {
        $this->totals();
        // recupero il simbolo della valuta
        $s = array("EUR","USD","GBP","JPY");
        $S = array("EUR"=>"&euro;","USD"=>"&#36;","GBP"=>"&pound;","JPY"=>"&yen;");
        $C = strtoupper((string)$this->cv);
        if(in_array($C,$s)){$symbol = $S[$C];}
        else{$symbol = "&curren;";}
        // crea la tabella bootstrap
        echo "\n<div class='table-responsive'>\n";
        echo "<table class='table'>\n\n";
        echo " <thead>\n";
        echo "  <tr>\n";
        echo "   <th>&#35;</th>\n";
        echo "   <th>Art.</th>\n";
        echo "   <th>Descrizione</th>\n";
        echo "   <th>Prezzo</th>\n";
        echo "   <th>Quantit&agrave;</th>\n";
        echo "   <th>Totale</th>\n";
        echo "   <th>IVA</th>\n";
        echo "   <th>action</th>\n";
        echo "  </tr>\n";
        echo " </thead>\n\n";
        echo " <tbody>\n";
        // conto gli articoli
        $k = count($this->items);
        for($i=0; $i<$k; $i++)
        {
            $unknow = "";
            $know = array('pos','art','desc','price','qty','row_amount','row_tax');
            foreach($this->items[$i] as $key => $v){if(!in_array($key, $know)){$unknow .= " - <strong>$key</strong>:$v ";}}
            
            echo "  <tr>\n";
            echo "   <td>".$this->items[$i]['pos']."</td>\n";
            echo "   <td>".$this->items[$i]['art']."</td>\n";
            echo "   <td>".$this->items[$i]['desc']."</td>\n";
            echo "   <td>".$symbol." ".number_format($this->items[$i]['price'],2,',','.')."</td>\n";
            echo "   <td>".$this->items[$i]['qty']."</td>\n";
            echo "   <td>".$symbol." ".number_format($this->items[$i]['row_amount'],2,',','.')."</td>\n";
            echo "   <td>".$symbol." ".number_format($this->items[$i]['row_tax'],2,',','.')."</td>\n";
            echo "   <td><a href='#'><span class='glyphicon glyphicon-edit'></span></a>";
            echo "   <a href='#'><span class='glyphicon glyphicon-trash'></span></a></td>\n";
            echo "  </tr>\n";
            echo "  <tr>\n";
            echo "   <td colspan='2'><small><strong>Note aggiuntive:</strong></small></td>";
            echo "   <td colspan='5'><small>$unknow</small></td>";
            echo "   <td><br><br></td>";
            echo "  </tr>\n";
        }
        echo "  <tr><td colspan='8'><br><br></td></tr>\n";
        echo "  <tr>\n";
        echo "   <td colspan='8'>Total cart amount: ".$symbol." ".number_format($this->tot_amount,2,',','.')."</td>\n";
        echo "  </tr>\n";
        echo "  <tr>\n";
        echo "   <td colspan='8'>Total cart tax: ".$symbol." ".number_format($this->tot_tax,2,',','.')."</td>\n";
        echo "  </tr>\n";
        echo "  <tr>\n";
        echo "   <td colspan='8'><strong>Total amount to pay: ".$symbol." ".number_format(($this->tot_amount + $this->tot_tax),2,',','.')."</strong></td>\n";
        echo "  </tr>\n";
        echo " </tbody>\n\n";
        echo "</table>\n";
        echo "</div>\n";
    }
    
    // restituisce un array con tutti i dati del carrello
    public function CartArray()
    {
        $data = array();
        $data['userID'] = $this->userID;
        $data['code'] = $this->code;
        $data['start'] = $this->start;
        $data['currency'] = $this->cv;
        $data['change'] = $this->change;
        $data['pay_method'] = $this->pay_method;
        $data['tax'] = $this->tax;
        $data['tot_amount'] = $this->tot_amount;
        $data['tot_tax'] = $this->tot_tax;
        $data['items'] = $this->items;
        return $data;
    }
}
//          END CLASS
//**************************************************
?>
